@@ -83,20 +83,23 @@ class TestAlternativeDocument(DocumentBase):
     """A (not very) alternative implementation of Document."""
 
 
+ALL_SCENARIOS = tests.LOCAL_DATABASES_SCENARIOS + [
+    ('http', {'make_database_for_test': make_http_database_for_test,
+              'copy_database_for_test': copy_http_database_for_test,
+              'make_document_for_test': tests.make_document_for_test,
+              'make_app_with_state': make_http_app}),
+    ('oauth_http', {'make_database_for_test':
+                    make_oauth_http_database_for_test,
+                    'copy_database_for_test':
+                    copy_oauth_http_database_for_test,
+                    'make_document_for_test': tests.make_document_for_test,
+                    'make_app_with_state': make_oauth_http_app})
+    ] + tests.C_DATABASE_SCENARIOS
+
+
 class AllDatabaseTests(tests.DatabaseBaseTests, tests.TestCaseWithServer):
 
-    scenarios = tests.LOCAL_DATABASES_SCENARIOS + [
-        ('http', {'make_database_for_test': make_http_database_for_test,
-                  'copy_database_for_test': copy_http_database_for_test,
-                  'make_document_for_test': tests.make_document_for_test,
-                  'make_app_with_state': make_http_app}),
-        ('oauth_http', {'make_database_for_test':
-                        make_oauth_http_database_for_test,
-                        'copy_database_for_test':
-                        copy_oauth_http_database_for_test,
-                        'make_document_for_test': tests.make_document_for_test,
-                        'make_app_with_state': make_oauth_http_app})
-        ] + tests.C_DATABASE_SCENARIOS
+    scenarios = ALL_SCENARIOS
 
     def test_close(self):
         self.db.close()
@@ -215,6 +218,13 @@ class AllDatabaseTests(tests.DatabaseBaseTests, tests.TestCaseWithServer):
             new_vc.is_newer(deleted_vc),
             "%s does not supersede %s" % (doc2.rev, deleted_doc.rev))
 
+    def test_delete_doesnt_open_the_door_to_random_revs(self):
+        doc = self.db.create_doc_from_json(simple_doc, doc_id='my_doc_id')
+        self.db.delete_doc(doc)
+        rand_rev_doc = self.make_document('my_doc_id', 'other:3', '{}')
+        self.assertRaises(errors.RevisionConflict, self.db.put_doc,
+                          rand_rev_doc)
+
     def test_get_doc_after_put(self):
         doc = self.db.create_doc_from_json(simple_doc, doc_id='my_doc_id')
         self.assertGetDoc(self.db, 'my_doc_id', doc.rev, simple_doc, False)
@@ -266,6 +276,29 @@ class AllDatabaseTests(tests.DatabaseBaseTests, tests.TestCaseWithServer):
     def test_get_docs_empty_list(self):
         self.assertEqual([], list(self.db.get_docs([])))
 
+    def test_get_all_docs_empty(self):
+        self.assertEqual([], list(self.db.get_all_docs()[1]))
+
+    def test_get_all_docs(self):
+        doc1 = self.db.create_doc_from_json(simple_doc)
+        doc2 = self.db.create_doc_from_json(nested_doc)
+        self.assertEqual(
+            sorted([doc1, doc2]), sorted(list(self.db.get_all_docs()[1])))
+
+    def test_get_all_docs_exclude_deleted(self):
+        doc1 = self.db.create_doc_from_json(simple_doc)
+        doc2 = self.db.create_doc_from_json(nested_doc)
+        self.db.delete_doc(doc2)
+        self.assertEqual([doc1], list(self.db.get_all_docs()[1]))
+
+    def test_get_all_docs_include_deleted(self):
+        doc1 = self.db.create_doc_from_json(simple_doc)
+        doc2 = self.db.create_doc_from_json(nested_doc)
+        self.db.delete_doc(doc2)
+        self.assertEqual(
+            sorted([doc1, doc2]),
+            sorted(list(self.db.get_all_docs(include_deleted=True)[1])))
+
     def test_handles_nested_content(self):
         doc = self.db.create_doc_from_json(nested_doc)
         self.assertGetDoc(self.db, doc.doc_id, doc.rev, nested_doc, False)
@@ -331,9 +364,38 @@ class AllDatabaseTests(tests.DatabaseBaseTests, tests.TestCaseWithServer):
         self.assertGetDoc(self.db, doc.doc_id, doc.rev, nested_doc, False)
 
 
+peculiar_json1 = '{  "a": 1 ,\n  "b": 2}'
+peculiar_json2 = '{"a": 1,"b":2}'
+peculiar_json3 = '{"a": 1,"b":2,\n"c":  3}'
+
+
+class OptionalJSONPreservationlDatabaseTests(tests.DatabaseBaseTests,
+                                           tests.TestCaseWithServer):
+
+    scenarios = ALL_SCENARIOS
+
+    def test_preserve_json_create_doc_from_json(self):
+        doc = self.db.create_doc_from_json(peculiar_json1)
+        doc1 = self.db.get_doc(doc.doc_id)
+        self.assertEqual(peculiar_json1, doc1.get_json())
+
+    def test_preserve_json_put_doc(self):
+        doc = self.make_document('doc-id', None, peculiar_json1)
+        doc = self.db.put_doc(doc)
+        doc1 = self.db.get_doc('doc-id')
+        self.assertEqual(peculiar_json1, doc1.get_json())
+        doc1.set_json(peculiar_json2)
+        self.db.put_doc(doc1)
+        doc2 = self.db.get_doc('doc-id')
+        self.assertEqual(peculiar_json2, doc2.get_json())
+
+
+LOCAL_SCENARIOS = tests.LOCAL_DATABASES_SCENARIOS + tests.C_DATABASE_SCENARIOS
+
+
 class DocumentSizeTests(tests.DatabaseBaseTests):
 
-    scenarios = tests.LOCAL_DATABASES_SCENARIOS + tests.C_DATABASE_SCENARIOS
+    scenarios = LOCAL_SCENARIOS
 
     def test_put_doc_refuses_oversized_documents(self):
         self.db.set_document_size_limit(1)
@@ -357,7 +419,7 @@ class DocumentSizeTests(tests.DatabaseBaseTests):
 
 class LocalDatabaseTests(tests.DatabaseBaseTests):
 
-    scenarios = tests.LOCAL_DATABASES_SCENARIOS + tests.C_DATABASE_SCENARIOS
+    scenarios = LOCAL_SCENARIOS
 
     def test_create_doc_different_ids_diff_db(self):
         doc1 = self.db.create_doc_from_json(simple_doc)
@@ -369,29 +431,6 @@ class LocalDatabaseTests(tests.DatabaseBaseTests):
         doc = self.make_document('/a', None, simple_doc)
         self.assertRaises(errors.InvalidDocId, self.db.put_doc, doc)
 
-    def test_get_all_docs_empty(self):
-        self.assertEqual([], list(self.db.get_all_docs()[1]))
-
-    def test_get_all_docs(self):
-        doc1 = self.db.create_doc_from_json(simple_doc)
-        doc2 = self.db.create_doc_from_json(nested_doc)
-        self.assertEqual(
-            sorted([doc1, doc2]), sorted(list(self.db.get_all_docs()[1])))
-
-    def test_get_all_docs_exclude_deleted(self):
-        doc1 = self.db.create_doc_from_json(simple_doc)
-        doc2 = self.db.create_doc_from_json(nested_doc)
-        self.db.delete_doc(doc2)
-        self.assertEqual([doc1], list(self.db.get_all_docs()[1]))
-
-    def test_get_all_docs_include_deleted(self):
-        doc1 = self.db.create_doc_from_json(simple_doc)
-        doc2 = self.db.create_doc_from_json(nested_doc)
-        self.db.delete_doc(doc2)
-        self.assertEqual(
-            sorted([doc1, doc2]),
-            sorted(list(self.db.get_all_docs(include_deleted=True)[1])))
-
     def test_get_all_docs_generation(self):
         self.db.create_doc_from_json(simple_doc)
         self.db.create_doc_from_json(nested_doc)
@@ -445,6 +484,23 @@ class LocalDatabaseTests(tests.DatabaseBaseTests):
         # strictly newer locally
         self.assertTrue(rev not in doc2.rev)
 
+    def test_put_doc_if_newer_autoresolve_deleted(self):
+        doc1 = self.db.create_doc_from_json('{}')
+        self.db.delete_doc(doc1)
+        rev = doc1.rev
+        doc = self.make_document(doc1.doc_id, "whatever:1", None)
+        state, _ = self.db._put_doc_if_newer(
+            doc, save_conflict=False, replica_uid='r', replica_gen=1,
+            replica_trans_id='foo')
+        self.assertEqual('superseded', state)
+        doc2 = self.db.get_doc(doc1.doc_id, include_deleted=True)
+        self.assertIsNone(doc2.get_json())
+        v2 = vectorclock.VectorClockRev(doc2.rev)
+        self.assertTrue(v2.is_newer(vectorclock.VectorClockRev("whatever:1")))
+        self.assertTrue(v2.is_newer(vectorclock.VectorClockRev(rev)))
+        # strictly newer locally
+        self.assertTrue(rev not in doc2.rev)
+
     def test_put_doc_if_newer_already_converged(self):
         orig_doc = '{"new": "doc"}'
         doc1 = self.db.create_doc_from_json(orig_doc)
@@ -600,9 +656,47 @@ class LocalDatabaseTests(tests.DatabaseBaseTests):
                          self.db.whats_changed(2))
 
 
+class LocalDatabaseOptionalJSONPreservationTests(tests.DatabaseBaseTests):
+
+    scenarios = LOCAL_SCENARIOS
+
+    def test_preserve_json_put_doc_if_newer(self):
+        doc = self.make_document('doc-id', 'other:1', peculiar_json1)
+        self.db._put_doc_if_newer(doc, save_conflict=False,
+                                  replica_uid="other", replica_gen=1)
+        doc1 = self.db.get_doc('doc-id')
+        self.assertEqual(peculiar_json1, doc1.get_json())
+        doc1 = self.make_document('doc-id', 'other:2', peculiar_json2)
+        self.db._put_doc_if_newer(doc1, save_conflict=False,
+                                  replica_uid="other", replica_gen=2)
+        doc2 = self.db.get_doc('doc-id')
+        self.assertEqual(peculiar_json2, doc2.get_json())
+
+    def test_preserve_json_put_doc_if_newer_autoresolve_json(self):
+        doc = self.make_document('doc-id', None, peculiar_json1)
+        self.db.put_doc(doc)
+        doc1 = self.make_document('doc-id', 'other:1', peculiar_json1)
+        self.db._put_doc_if_newer(doc1, save_conflict=False,
+                                  replica_uid="other", replica_gen=1)
+        doc2 = self.db.get_doc('doc-id')
+        self.assertEqual(peculiar_json1, doc2.get_json())
+
+    def test_preserve_json_put_doc_if_newer_autoresolve_content(self):
+        if self.db.__class__.__name__ == 'CDatabase':
+            self.skipTest("the C implementation atm uses string "
+                          "equivalence only")
+        doc = self.make_document('doc-id', None, peculiar_json1)
+        self.db.put_doc(doc)
+        doc1 = self.make_document('doc-id', 'other:1', peculiar_json2)
+        self.db._put_doc_if_newer(doc1, save_conflict=False,
+                                  replica_uid="other", replica_gen=1)
+        doc2 = self.db.get_doc('doc-id')
+        self.assertEqual(peculiar_json2, doc2.get_json())
+
+
 class LocalDatabaseValidateGenNTransIdTests(tests.DatabaseBaseTests):
 
-    scenarios = tests.LOCAL_DATABASES_SCENARIOS + tests.C_DATABASE_SCENARIOS
+    scenarios = LOCAL_SCENARIOS
 
     def test_validate_gen_and_trans_id(self):
         self.db.create_doc_from_json(simple_doc)
@@ -626,7 +720,7 @@ class LocalDatabaseValidateGenNTransIdTests(tests.DatabaseBaseTests):
 
 class LocalDatabaseValidateSourceGenTests(tests.DatabaseBaseTests):
 
-    scenarios = tests.LOCAL_DATABASES_SCENARIOS + tests.C_DATABASE_SCENARIOS
+    scenarios = LOCAL_SCENARIOS
 
     def test_validate_source_gen_and_trans_id_same(self):
         self.db._set_replica_gen_and_trans_id('other', 1, 'T-sid')
@@ -646,7 +740,7 @@ class LocalDatabaseValidateSourceGenTests(tests.DatabaseBaseTests):
 class LocalDatabaseWithConflictsTests(tests.DatabaseBaseTests):
     # test supporting/functionality around storing conflicts
 
-    scenarios = tests.LOCAL_DATABASES_SCENARIOS + tests.C_DATABASE_SCENARIOS
+    scenarios = LOCAL_SCENARIOS
 
     def test_get_docs_conflicted(self):
         doc1 = self.db.create_doc_from_json(simple_doc)
@@ -1012,9 +1106,30 @@ class LocalDatabaseWithConflictsTests(tests.DatabaseBaseTests):
         self.assertRaises(errors.ConflictedDoc, self.db.delete_doc, doc2)
 
 
+class LocalDatabaseWithConflictsOptionalJSONPreservationTests(
+    tests.DatabaseBaseTests):
+    # test supporting/functionality around storing conflicts
+
+    scenarios = LOCAL_SCENARIOS
+
+    def test_json_preservation_put_doc_if_newer_save_conflicted(self):
+        doc1 = self.db.create_doc_from_json(peculiar_json1)
+        # Document is inserted as a conflict
+        doc2 = self.make_document(doc1.doc_id, 'alternate:1',
+                                  peculiar_json3)
+        state, _ = self.db._put_doc_if_newer(
+            doc2, save_conflict=True, replica_uid='r', replica_gen=1,
+            replica_trans_id='foo')
+        self.assertEqual('conflicted', state)
+        # check json in db
+        conflicts = self.db.get_doc_conflicts(doc1.doc_id)
+        self.assertEqual(peculiar_json3, conflicts[0].get_json())
+        self.assertEqual(peculiar_json1, conflicts[1].get_json())
+
+
 class DatabaseIndexTests(tests.DatabaseBaseTests):
 
-    scenarios = tests.LOCAL_DATABASES_SCENARIOS + tests.C_DATABASE_SCENARIOS
+    scenarios = LOCAL_SCENARIOS
 
     def assertParseError(self, definition):
         self.db.create_doc_from_json(nested_doc)
@@ -1098,6 +1213,9 @@ class DatabaseIndexTests(tests.DatabaseBaseTests):
         self.db.delete_index('test-idx')
         self.assertEqual([], self.db.list_indexes())
 
+    def test_delete_index_does_not_fail_if_no_index(self):
+        self.db.delete_index('foo')
+
     def test_create_adds_to_index(self):
         self.db.create_index('test-idx', 'key')
         doc = self.db.create_doc_from_json(simple_doc)
@@ -1155,6 +1273,15 @@ class DatabaseIndexTests(tests.DatabaseBaseTests):
         docs = self.db.get_from_index('test-idx', 'value', 'value2-1')
         self.assertTrue(docs[0].has_conflicts)
 
+    def test_get_from_index_wilcard_vs_non_wildcard(self):
+        docs = {}
+        for v in ('a', 'b', 'b1', 'b2', 'c', 'c1', 'c2', 'd'):
+            docs[v] = self.db.create_doc_from_json(json.dumps({"key": v}))
+        self.db.create_index('test-idx', 'key')
+        self.assertEqual([docs['c']], self.db.get_from_index('test-idx', 'c'))
+        self.assertEqual([docs['c'], docs['c1'], docs['c2']],
+                         self.db.get_from_index('test-idx', 'c*'))
+
     def test_get_index_keys_multi_list_list(self):
         self.db.create_doc_from_json(
             '{"key": "value1-1 value1-2 value1-3", '
@@ -1347,6 +1474,17 @@ class DatabaseIndexTests(tests.DatabaseBaseTests):
             [doc4, doc3, doc2],
             self.db.get_range_from_index('test-idx', None, ('value2', '*')))
 
+    def test_get_wilcard_range_vs_non_wildcard(self):
+        docs = {}
+        for v in ('a', 'b', 'b1', 'b2', 'c', 'c1', 'c2', 'd'):
+            docs[v] = self.db.create_doc_from_json(json.dumps({"key": v}))
+        self.db.create_index('test-idx', 'key')
+        self.assertEqual([docs['b'], docs['b1'], docs['b2'], docs['c']],
+                         self.db.get_range_from_index('test-idx', 'b', 'c'))
+        self.assertEqual([docs['b'], docs['b1'], docs['b2'], docs['c'],
+                          docs['c1'], docs['c2']],
+                         self.db.get_range_from_index('test-idx', 'b', 'c*'))
+
     def test_get_range_from_index_illegal_wildcard_order(self):
         self.db.create_index('test-idx', 'k1', 'k2')
         self.assertRaises(
@@ -1829,6 +1967,9 @@ class PythonBackendTests(tests.DatabaseBaseTests):
         doc = self.db.create_doc(self.simple_doc, doc_id='my_doc_id')
         self.assertTrue(isinstance(doc, TestAlternativeDocument))
 
+    def test_create_doc_with_invalid_content(self):
+        self.assertRaises(errors.InvalidContent, self.db.create_doc, "{}")
+
     def test_get_doc_after_put_with_factory(self):
         doc = self.db.create_doc(self.simple_doc, doc_id='my_doc_id')
         self.db.set_document_factory(TestAlternativeDocument)
